/**
 * Copyright (c) Meta Platforms, Inc. and its affiliates. All Rights Reserved.
 */

'use strict';

var fs = require('fs');
var mic = require('mic');
var readline = require('readline');

var AUDIO_PATH = '/tmp/output.raw';
var MIC_TIMEOUT_MS = 3000;

module.exports = function (wit, handleResponse, context) {
  var rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout
  });
  rl.setPrompt('> ');

  var prompt = function prompt() {
    rl.prompt();
    rl.write(null, { ctrl: true, name: 'e' });
  };
  prompt();

  var makeResponseHandler = function makeResponseHandler(rsp) {
    if (handleResponse) {
      handleResponse(rsp);
    } else {
      console.log(JSON.stringify(rsp));
    }
    prompt();
  };

  rl.on('line', function (line) {
    line = line.trim();
    if (!line) {
      return prompt();
    }

    // POST /speech
    if (line === '!speech') {
      var microphone = mic({
        bitwidth: '16',
        channels: '1',
        encoding: 'signed-integer',
        endian: 'little',
        fileType: 'raw',
        rate: '16000'
      });

      var inputAudioStream = microphone.getAudioStream();
      var outputFileStream = fs.WriteStream(AUDIO_PATH);
      inputAudioStream.pipe(outputFileStream);

      inputAudioStream.on('startComplete', function () {
        setTimeout(function () {
          microphone.stop();
        }, MIC_TIMEOUT_MS);
      });
      inputAudioStream.on('stopComplete', function () {
        var stream = fs.ReadStream(AUDIO_PATH);
        wit.speech('audio/raw;encoding=signed-integer;bits=16;rate=16000;endian=little', stream, context).then(makeResponseHandler).catch(console.error);
      });

      microphone.start();

      return;
    }

    // GET /message
    wit.message(line, context).then(makeResponseHandler).catch(console.error);
  });
};